/* pwrite.c     write out data as a POT format file    -jpb 6/25/95    */
/*              read data in from POT file  -jpb 7/15/95 */

/* pwrite.c   uses routines from the gd 1.1.1 library:   

   gd 1.1.1 is copyright 1994, Quest Protein Database Center, Cold Spring
   Harbor Labs. Permission granted to copy and distribute this work
   provided that this notice remains intact.

	Tom Boutell, boutell@netcom.com
	http://sunsite.unc.edu/boutell/index.html
	116 14th Ave E Apt 2
	Seattle, WA 98102
*/

#include <stdio.h>
#include <stdlib.h>
#include <gd.h>
#include <gdfontl.h>
#include <gdfonts.h>
#include "hl.h"

extern int sigbits;        /* number of significant bits to save */

/* ---------------------------------------------------------------------- */
/* pot_write()  write out data in hf array (xsize x ysize) as a greyscale */
/*              POT format image file with the name <fname>.              */
/*              hf is assumed hold values between 0.0 and 1.0             */
/* ---------------------------------------------------------------------- */

int pot_write(hfield hf, char *fname)
{
    int xsize, ysize;
			/* Input and output files */
    FILE *out;
    gdImagePtr im_out;       /* Input and output images */
    int blue;                   /* Color indexes */
    int red;
    int green;
    int x,y;
    int color;
    unsigned char onebit, mask_lo;

    xsize = hf.xsize;
    ysize = hf.ysize;
      /* Create output image, x by y pixels. */
    im_out = gdImageCreate(xsize*2,ysize);

	/* First color allocated is background. */
	/* this loop makes a 0-255 graylevel palette (only 64 unique tho) */
    for (x = 0; x < 256; x++) {  
	  red = x - 2;
	  if (red<0) red = 0;
	  blue = x - 3;
	  if (blue<0) blue=0;
	  green = x;
	  gdImageColorAllocate(im_out, red,green,blue);
     }

     if (sigbits > 8) { /* write 16-bit POT file */
      
      onebit = 128;   /* binary 10000000  */
      mask_lo = 0;
      for(y=0;y<(sigbits-8);y++) {  /* build up mask byte */
	mask_lo ^= onebit;
	onebit >>= 1;
      }
 
      for(y=0;y<ysize;y++) {
	for(x=0;x<xsize;x++) {
	  color = (unsigned int) (65535.999 * El(hf.a,x,y));
	  im_out->pixels[x][y] = color >> 8; 
	  im_out->pixels[x+xsize][y] = (color % 256) & mask_lo; 
	} /* for x */
      }   /* for y */

     } else {  /* 8 bits or less precision */
      
      onebit = 128;   /* binary 10000000  */
      mask_lo = 0;
      for(y=0;y<(sigbits);y++) {  /* build up mask byte */
	mask_lo ^= onebit;
	onebit >>= 1;
      }
 
      for(y=0;y<ysize;y++) {
	for(x=0;x<xsize;x++) {
	  color = (unsigned int) (65535.999 * El(hf.a,x,y));
	  im_out->pixels[x][y] = (color >> 8) & mask_lo; 
	  im_out->pixels[x+xsize][y] = 0; 
	}
      }

     } /* if sigbits */

     out = fopen(fname, "wb");
	
	/* Write GIF */
     gdImageGif(im_out, out);
     fclose(out);
     gdImageDestroy(im_out);         /* free up image memory */
     return(0);
} /* pot_write() */

/* read hf array from POT */

int pot_read(hfield *hfp, char *fname) 
{
  hfield hf;
  int xsize,ysize;
  int ix, iy;
  gdImagePtr im;
  FILE *infile;

  infile = fopen(fname, "rb");
  if (infile == NULL) {
    fprintf(stderr,"Trouble opening POT file %s\n",fname);
    return(1);
  }
  im = gdImageCreateFromGif(infile);
  fclose(infile);

  if (!im) {
    fprintf(stderr,"Trouble reading POT file %s\n",fname);
    return(1);
  }

  xsize = gdImageSX(im) / 2;
  ysize = gdImageSY(im);
  
  if (xsize*2 != gdImageSX(im)) {
    printf("Error: file cannot be POT, horizontal dimension is odd.\n");
    return(1);
  }

  if (h_newr(&hf,xsize,ysize,fname)) return(1);

  for (ix = 0; ix < xsize; ix++) {
    for (iy = 0; iy < ysize; iy++) {
      El(hf.a,ix,iy) = (PTYPE) im->pixels[ix][iy]/256.0 + 
	 (PTYPE) im->pixels[ix+xsize][iy]/65536.0;
    }
  }

  *hfp = hf;
  return(0);
} /* end pot_read() */
